codeunit 50101 "Remote JobQueue Runner"
{
    trigger OnRun()
    var
        ErrDescription: Text;
    begin
        IF RunRemoteJobQueue('Cronus', CODEUNIT::"Custom Codeunit", ErrDescription) THEN
            MESSAGE('sucess')
        ELSE
            ERROR('Run Codeunit remotly failed : \\' + ErrDescription);
    end;

    procedure RunRemoteJobQueue(CompanyName: Text; CodeunitID: Integer; WaitCompletion: Boolean; VAR ErrDescription: Text): Boolean
    var
        RecID: RecordId;
    begin
        // Just run with empty "Record ID To Process" parameter
        EXIT(RunRemoteJobQueueRecordID(CompanyName, CodeunitID, RecID, WaitCompletion, ErrDescription));
    end;

    procedure RunRemoteJobQueueRecordID(CompanyName: Text; CodeunitID: Integer; RecordIDToProcess: RecordID; WaitCompletion: Boolean; VAR ErrDescription: Text): Boolean
    var
        JobQueue: Record "Job Queue Entry";
        JobQueueLog: Record "Job Queue Log Entry";
        StartDateTime: DateTime;
        JobQueueID: Guid;
    begin
        ErrDescription := '';

        JobQueue.CHANGECOMPANY(CompanyName);
        JobQueue.INIT();
        JobQueue.ID := CREATEGUID();
        JobQueueID := JobQueue.ID; // store for waiting/error handling
        JobQueue."Object Type to Run" := JobQueue."Object Type to Run"::Codeunit;
        JobQueue."Object ID to Run" := CodeunitID;
        JobQueue."Record ID to Process" := RecordIDToProcess;
        JobQueue."Recurring Job" := FALSE;
        JobQueue."Earliest Start Date/Time" := CREATEDATETIME(TODAY, TIME);
        JobQueue.Status := JobQueue.Status::Ready;
        JobQueue."User ID" := USERID;
        JobQueue.INSERT();
        JobQueue.SetStatus(JobQueue.Status::Ready);

        // Enqueue task manually to target particular company
        JobQueue."System Task ID" := TASKSCHEDULER.CREATETASK(
            CODEUNIT::"Job Queue Dispatcher",
            CODEUNIT::"Job Queue Error Handler",
            TRUE,
            CompanyName,
            JobQueue."Earliest Start Date/Time",
            JobQueue.RECORDID);
        JobQueue.MODIFY(FALSE);

        if not WaitCompletion then exit;
        COMMIT;

        StartDateTime := CREATEDATETIME(TODAY, TIME);

        // Wait until job queue completion
        WHILE (TRUE) DO BEGIN
            SLEEP(100);
            // Job Queue do not exists anymore = finish sucessfully :
            IF NOT JobQueue.GET(JobQueueID) THEN EXIT(TRUE);
            // Job Queue in Error :
            IF JobQueue.Status = JobQueue.Status::Error THEN BEGIN
                // get error from log
                JobQueueLog.SETRANGE(ID, JobQueueID);
                IF JobQueueLog.FINDLAST THEN
                    ErrDescription := JobQueueLog."Error Message";
                EXIT(FALSE);
            END;

            // Give 15s to taskscheduler to start the job
            IF JobQueue.Status = JobQueue.Status::Ready THEN
                IF CREATEDATETIME(TODAY, TIME) - StartDateTime > 15000 THEN BEGIN
                    ErrDescription := 'The task scheduler didn''t launch the task within allowed timeout';
                    EXIT(FALSE);
                END;
        END;

        EXIT(FALSE);
    end;
}